@@ -29,7 +29,7 @@ from zope.app.testing import placelesssetup
 from zope.container import contained, btree
 from zope.testing import module, renormalizing
 
-from mongopersist import datamanager, interfaces, serialize
+from mongopersist import datamanager, interfaces, serialize, testing
 from mongopersist.zope import container
 
 class ApplicationRoot(container.SimpleMongoContainer):
@@ -54,6 +54,131 @@ class Person(container.MongoContained, SimplePerson):
     pass
 
 
+class Campaigns(container.MongoContainer):
+
+    _m_collection = 'campaigns'
+
+    def add(self, campaign):
+        self[campaign.name] = campaign
+
+
+class MongoItem(container.MongoContained,
+                persistent.Persistent):
+    pass
+
+class Campaign(MongoItem, container.MongoContainer):
+
+    _m_collection = 'persons'
+    _p_mongo_collection = 'campaigns'
+
+    def __init__(self, name):
+        self.name = name
+
+    def __repr__(self):
+        return '<%s %s>' %(self.__class__.__name__, self.name)
+
+
+def doctest_Campaign():
+    """MongoContainer: basic
+
+    Let's make sure events are fired correctly:
+
+      >>> zope.component.provideHandler(handleObjectModifiedEvent)
+
+    Let's add a container to the root:
+
+      >>> transaction.commit()
+      >>> dm.root['c'] = Campaigns()
+
+      >>> db = dm._conn[DBNAME]
+      >>> pprint(list(db['campaigns'].find()))
+      []
+
+      [{u'_id': ObjectId('4e7ddf12e138237403000000'),
+        u'_m_collection': u'person'}]
+
+    It is unfortunate that the '_m_collection' attribute is set. This is
+    avoidable using a sub-class.
+
+      >>> dm.root['c'][u'one'] = Campaign(u'one')
+      ContainerModifiedEvent: <...Campaigns ...>
+      >>> dm.root['c'].keys()
+      [u'one']
+      >>> dm.root['c'][u'one']
+      <Campaign one>
+
+      >>> dm.root['c']['one'].__parent__
+      <mongopersist.zope.tests.test_container.Campaigns object at 0x001122>
+      >>> dm.root['c']['one'].__name__
+      u'one'
+
+    It is a feature of the container that the item is immediately available
+    after assignment, but before the data is stored in the database. Let's
+    commit and access the data again:
+
+      >>> transaction.commit()
+
+      >>> pprint(list(db['campaigns'].find()))
+      [{u'_id': ObjectId('4e7ddf12e138237403000000'),
+        u'key': u'one',
+        u'name': u'one',
+        u'parent': DBRef(u'mongopersist.zope.tests.test_container.Campaigns',
+            ObjectId('4e7ddf12e138237403000000'),
+            u'mongopersist_container_test')}]
+
+      >>> 'one' in dm.root['c']
+      True
+      >>> dm.root['c'].keys()
+      [u'one']
+      >>> dm.root['c']['one'].__parent__
+      <mongopersist.zope.tests.test_container.Campaigns object at 0x001122>
+      >>> dm.root['c']['one'].__name__
+      u'one'
+
+    We get a usual key error, if an object does not exist:
+
+      >>> dm.root['c']['roy']
+      Traceback (most recent call last):
+      ...
+      KeyError: 'roy'
+
+      >>> 'roy' in dm.root['c']
+      False
+
+    Now remove the item:
+
+      >>> del dm.root['c']['one']
+      ContainerModifiedEvent: <...Campaigns ...>
+
+    The changes are immediately visible.
+
+      >>> dm.root['c'].keys()
+      []
+      >>> dm.root['c']['one']
+      Traceback (most recent call last):
+      ...
+      KeyError: 'one'
+
+    Make sure it is really gone after committing:
+
+      >>> transaction.commit()
+      >>> dm.root['c'].keys()
+      []
+
+    Check adding of more objects:
+
+      >>> dm.root['c'][u'1'] = c1 = Campaign(u'One')
+      ContainerModifiedEvent: <...Campaigns ...>
+      >>> dm.root['c'][u'2'] = c2 = Campaign(u'Two')
+      ContainerModifiedEvent: <...Campaigns ...>
+      >>> dm.root['c'][u'3'] = Campaign(u'Three')
+      ContainerModifiedEvent: <...Campaigns ...>
+
+      >>> sorted(dm.root['c'].keys())
+      [u'1', u'2', u'3']
+    """
+
+
 def doctest_SimpleMongoContainer_basic():
     """SimpleMongoContainer: basic
 
@@ -144,6 +269,19 @@ def doctest_SimpleMongoContainer_basic():
 
       >>> pprint(list(db['person'].find()))
       []
+
+    Check adding of more objects:
+
+      >>> dm.root['c'][u'roy'] = SimplePerson(u'Roy')
+      ContainerModifiedEvent: <...SimpleMongoContainer ...>
+      >>> dm.root['c'][u'adam'] = SimplePerson(u'Adam')
+      ContainerModifiedEvent: <...SimpleMongoContainer ...>
+      >>> dm.root['c'][u'marius'] = SimplePerson(u'Marius')
+      ContainerModifiedEvent: <...SimpleMongoContainer ...>
+
+      >>> sorted(dm.root['c'].keys())
+      [u'adam', u'marius', u'roy']
+
     """
 
 
@@ -231,6 +369,18 @@ def doctest_MongoContainer_basic():
       >>> transaction.commit()
       >>> dm.root['c'].keys()
       []
+
+    Check adding of more objects:
+
+      >>> dm.root['c'][u'roy'] = SimplePerson(u'Roy')
+      ContainerModifiedEvent: <...MongoContainer ...>
+      >>> dm.root['c'][u'adam'] = SimplePerson(u'Adam')
+      ContainerModifiedEvent: <...MongoContainer ...>
+      >>> dm.root['c'][u'marius'] = SimplePerson(u'Marius')
+      ContainerModifiedEvent: <...MongoContainer ...>
+
+      >>> sorted(dm.root['c'].keys())
+      [u'adam', u'marius', u'roy']
     """
 
 def doctest_MongoContainer_constructor():
@@ -264,6 +414,7 @@ def doctest_MongoContainer_constructor():
       >>> c._m_parent_key
       'site'
     """
+
 def doctest_MongoContainer_m_parent_key_value():
     r"""MongoContainer: _m_parent_key_value()
 
@@ -715,19 +866,17 @@ def setUp(test):
 
     # silence this, otherwise half-baked objects raise exceptions
     # on trying to __repr__ missing attributes
-    test.orig_DEBUG_EXCEPTION_FORMATTER = exceptionformatter.DEBUG_EXCEPTION_FORMATTER
+    test.orig_DEBUG_EXCEPTION_FORMATTER = \
+        exceptionformatter.DEBUG_EXCEPTION_FORMATTER
     exceptionformatter.DEBUG_EXCEPTION_FORMATTER = 0
 
 def tearDown(test):
     placelesssetup.tearDown(test)
     module.tearDown(test)
     test.globs['conn'].disconnect()
-    serialize.SERIALIZERS.__init__()
-    serialize.OID_CLASS_LRU.__init__(20000)
-    serialize.COLLECTIONS_WITH_TYPE.__init__()
-    serialize.AVAILABLE_NAME_MAPPINGS.__init__()
-    serialize.PATH_RESOLVE_CACHE.__init__()
-    exceptionformatter.DEBUG_EXCEPTION_FORMATTER = test.orig_DEBUG_EXCEPTION_FORMATTER
+    testing.resetCaches()
+    exceptionformatter.DEBUG_EXCEPTION_FORMATTER = \
+        test.orig_DEBUG_EXCEPTION_FORMATTER
 
 def test_suite():
     return doctest.DocTestSuite(
